// $Id: date.C,v 1.2 2001/04/12 15:26:19 dvermeir Exp $
// sys/time.h is needed for timeval
#define __EXTENSIONS__
#include <sys/time.h>
#include <time.h>

#include        <string.h> // strlen()
#include        <iostream>

#include        "date.h"

// this routine is taken from gnu fileutils-3.16/lib
// it is in the public domain

extern "C" {
time_t get_date(const char*,time_t*);
}

const string DateError::NAME("DateError");

Date::Date() {
struct  timeval t;
/*
*       t.tv_sec:       secs since 1/1/1970
*       t.tv_usec:      microseconds
*/
gettimeofday(&t, 0);
time_ = t.tv_sec;
}

Date::Date(time_t t): time_(t) {
}

Date::Date(unsigned int year, unsigned int month, unsigned int day,
            unsigned int hrs=0, unsigned int min=0, unsigned int sec=0): time_(0) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_sec       = sec;
tm.tm_min       = min;
tm.tm_hour      = hrs-1; // don't ask; it just works that way
tm.tm_mday      = day;
tm.tm_mon       = month;
tm.tm_year      = year-1900;
time_           = mktime(&tm);
}

unsigned short
Date::year() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_year + 1900;
}

unsigned short
Date::month() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_mon;
}

unsigned short
Date::day() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_mday;
}

unsigned short
Date::hours() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_hour;
}

unsigned short
Date::minutes() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_min;
}

unsigned short
Date::seconds() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_sec;
}

unsigned short
Date::wday() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_wday;
}

unsigned short
Date::yday() const {
struct tm       tm;
localtime_r(&time_,&tm);
return tm.tm_yday;
}

void
Date::year(unsigned short yr) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_year = yr - 1900;
time_   = mktime(&tm);
}

void
Date::month(unsigned short mo) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_mon = mo;
time_   = mktime(&tm);
}

void
Date::day(unsigned short dy) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_mday = dy;
time_   = mktime(&tm);
}

void
Date::hours(unsigned short hr) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_hour = hr;
time_   = mktime(&tm);
}

void
Date::minutes(unsigned short mins) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_min = mins;
time_   = mktime(&tm);
}

void
Date::seconds(unsigned short secs) {
struct tm       tm;
localtime_r(&time_,&tm);
tm.tm_sec = secs;
time_   = mktime(&tm);
}

string
Date::str(const char* fmt) const {
static const char*      d_fmt = "%a %b %d %H:%M:%S %Y";
// %Z, i.e. time zone abbreviation, not included in the default
if (fmt==0)
  fmt = d_fmt;
const int size = strlen(fmt)*20;
char* buf = new char[size+1];

struct tm       tm;
localtime_r(&time_,&tm);

strftime(buf, size, fmt, &tm);
string  s(buf);
delete buf;
return s;
}

string
Date::gmt(const char* fmt) const {
static const char*      d_fmt = "%a %b %d %H:%M:%S %Z %Y";

if (fmt==0)
  fmt = d_fmt;
const int size = strlen(fmt)*20;
char* buf = new char[size+1];

struct tm       tm;
gmtime_r(&time_,&tm);

strftime(buf, size, fmt, &tm);
string  s(buf);
delete buf;
return s;
}


// Creates a date from a string according to the syntax accepted by parse().
Date::Date (const string& s) throw(DateError)
{
static const string error(": cannot parse \"");
time_ = get_date(s.c_str(),0);
if (time_<0)
  throw DateError(error+s+"\"");
}

void
Date::time(time_t t) {
time_ = t;
}


