// $Id: datepattern.C,v 1.2 2001/04/12 15:26:19 dvermeir Exp $
#include <stdlib.h> // for atoi()
#include <ctype.h> // for isdigit()
#include "datepattern.h"

// Construct DatePattern containing first n components of p.
DatePattern::DatePattern(const DatePattern& p,unsigned int n) {
unsigned int m(min(p.size(),n));
date_.resize(m);
for (unsigned int i=0; i<n; ++i)
  date_[i] = p[i];
}


// Parses 1999-03-12 10:00, 1999-3-12, 1999-3, 1999
bool
DatePattern::parse(const string& s) {
date_.clear();
/* We use a simple brute-force procedure: copy the string
   to a C-string where we replace all non-digit chars with
   spaces. The repeatedly call strtol to parse this C string
   into a number of integers. See ``man strtol'' for details.
*/
char    cs[s.size()+1];

for (unsigned int i=0; i<s.size(); ++i)
  if (isdigit(s[i]))
     cs[i] = s[i];
  else
     cs[i] = ' ';
cs[s.size()] = '\0';

const char*     start(cs);
char*           end(cs);
unsigned int    n(0);

do {
  start = end;
  long l(strtol(start,&end,10));
  if (end!=start) {
    ++n;
    if (l<0 || n>5) { // no negative stuff and no more than 5 numbers expected
      date_.clear();
      return false;
      }
    if (n<5) // only 4 numbers stored.
      date_.push_back(static_cast<unsigned int>(l));
    }
  }
while (end!=start);
return true;
}

void
DatePattern::set(unsigned int year, unsigned int month, 
        unsigned int day, unsigned int hours) {
date_.resize(4);
date_[0] = year;
date_[1] = month;
date_[2] = day;
date_[3] = hours;
}

ostream&
operator<<(ostream& os,const DatePattern& date) {
unsigned int sz(date.size());
// Print in format year-month-day hours:00.
if (sz>0) {
  os << date[0] ;
  if (sz>1) {
    os << "-" << date[1];
    if (sz>2) {
      os << "-" << date[2];
      if (sz>3)
        os << " " << date[3] << ":00";
      }
    }
  }
return os;
}

