// $Id: expression.C,v 1.1 2001/04/12 12:27:43 dvermeir Exp $

#include <algorithm> // for equal()
#include "expression.h"

ostream&
operator<<(ostream& os,const Expression& e) {
e.print(os);
return os;
}

bool
PathExpression::eval(const LogRecord& r) const {
const Path& pr(r.path());
if (pr.size() < path_.size())
  return false;
return equal(path_.begin(), path_.end(), pr.begin());
}

void
PathExpression::print(ostream& os) const {
os << "path " << path_;
}

bool
DomainExpression::eval(const LogRecord& r) const {
const Domain& d(r.domain());
if (d.size() < domain_.size())
  return false;
return equal(domain_.begin(), domain_.end(), d.begin());
}

void
DomainExpression::print(ostream& os) const {
os << "domain " << domain_;
}

bool
DateExpression::eval(const LogRecord& r) const {
const DatePattern& d(r.date());
if (d.size() < date_.size())
  return false;
return equal(date_.begin(), date_.end(), d.begin());
};

void
DateExpression::print(ostream& os) const {
os << "date " << date_;
}

void
NotExpression::print(ostream& os)  const{
os << "exclude " << *exp_ ;
}

AndExpression::~AndExpression() {
for (list<Expression*>::const_iterator i=exps_.begin(); i!=exps_.end(); ++i)
  delete *i;
}

void
AndExpression::print(ostream& os) const {
for (list<Expression*>::const_iterator i=exps_.begin(); i!=exps_.end(); ++i)
  os << **i << "\n";
}

bool
AndExpression::eval(const LogRecord& r) const { 
for (list<Expression*>::const_iterator i=exps_.begin(); i!=exps_.end(); ++i)
  if (!(*i)->eval(r))
    return false;
return true;
}
