#ifndef EXPRESSION_H
#define EXPRESSION_H
// $Id: expression.h,v 1.2 2001/04/12 15:26:19 dvermeir Exp $

#include <list>

#include "path.h"
#include "domain.h"
#include "datepattern.h"
#include "logrecord.h"

class Expression {
public:
  Expression() {}
  virtual ~Expression() {}
  virtual bool eval(const LogRecord& r) const = 0;
  virtual void print(ostream& os) const = 0;
  friend ostream& operator<<(ostream& os,const Expression& e);
};

class PathExpression: public Expression {
public:
  PathExpression(const Path& path): path_(path) {}
  bool eval(const LogRecord& r) const;
  void print(ostream& os) const;
private:
  Path  path_;
};

class DomainExpression: public Expression {
public:
  DomainExpression(const Domain& domain): domain_(domain) {}
  bool eval(const LogRecord& r) const;
  void print(ostream& os) const;
private:
  Domain domain_;
};

class DateExpression: public Expression {
public:
  DateExpression(const DatePattern& date): date_(date) {}
  bool eval(const LogRecord& r) const;
  void print(ostream& os) const;
private:
  DatePattern date_;
};

class NotExpression: public Expression {
public:
  NotExpression(const Expression* e): exp_(e) {}
  ~NotExpression() { delete exp_; }
  bool eval(const LogRecord& r) const { return (exp_ ? !exp_->eval(r) : false); }
  void print(ostream& os) const;
private:
  const Expression* exp_;

};

class AndExpression: public Expression {
public:
  AndExpression() {}
  ~AndExpression();
  void add(Expression* e) { if (e) exps_.push_back(e); }
  bool eval(const LogRecord& r) const;
  void print(ostream& os) const;
private:
  list<Expression*> exps_;
};

#endif
