// $Id: httpstats.C,v 1.2 2001/04/12 15:26:19 dvermeir Exp $

#include <string>
#include <fstream>

// Httpstats main function.

static const string usage("httpstats configfile logfile");

#include "configuration.h"
#include "logrecord.h"
#include "stats.h"

int 
main(int argc,char* argv[]) {
// We put the lot in a try .. catch block, in case something throws
// an exception, which should not happen but..
try {
  // 1. check arguments: there must be exactly 2 of them.
  if (argc!=3) {
    cerr << "Usage: " << usage << endl;
    return 1; 
    }
  
  string config_file_name(argv[1]);
  string log_file_name(argv[2]);
  
  // 2. parse confiuration file. We will use a class Configuration that
  // will contain the configuration information. 
  // But first, we attempt to open the file.
  ifstream config_file(config_file_name.c_str());
  if (!config_file) {
    cerr << "Cannot open configuration file \"" << config_file_name << "\"" << endl;
    return 1;
    }
  
  // Then we try to parse.
  Configuration config; 
  if (!config.parse(config_file)) {
    cerr << "Fatal error in configuration file \"" << config_file_name << "\"" << endl;
    return 1;
    }
  
  cerr << config << endl;
  // 3. Process the log file line by line and process each line using config.
  ifstream log_file(log_file_name.c_str());
  if (!log_file) {
    cerr << "Cannot open log file \"" << log_file_name << "\"" << endl;
    return 1;
    }
  
  Stats             stats(config);  // A stats object must know about config.
  const Expression* select(config.select()); // Records must satisfy this to get into stats.
  string            line; //  Buffer for lines from the log_file.
  LogRecord         logrecord; // A line will be parsed into this object.
  int               n_lines_parsed(0); // Number of lines succesfully parsed.
  int               n_lines_selected(0); // Number of lines parsed and selected.
  int               n_error_lines(0); // Number of lines with errors.
  
  // Processing log file..

  while (getline(log_file,line)) {
    if (logrecord.parse_line(line)) {
      if (select) {
        if (select->eval(logrecord)) {
          ++n_lines_selected;
          stats.add(logrecord);
          }
        }
      else {
        stats.add(logrecord);
        ++n_lines_selected;
        }
      ++n_lines_parsed;
      }
    else
      ++n_error_lines;
    }
  
  cerr << "// Generate statistics output." << endl;
  cout << stats;
  
  // Show something on cerr to keep the user amused.
  cerr << n_lines_parsed << " lines processed, " 
       << n_lines_selected << " lines selected, " 
       << n_error_lines << " lines had errors" << endl;
  }
catch (exception& e) {
  cerr << e.what() << endl;
  }
}

