// $Id: path.C,v 1.1 2001/04/12 12:27:44 dvermeir Exp $
#include "path.h"

// Construct path containing the first n components of p.
Path::Path(const Path& p,unsigned int n) {
unsigned int m(min(p.size(),n));
path_.resize(m);
for (unsigned int i=0; i<m; ++i) 
  path_[i] = p[i];
}

bool
Path::parse(const string& ps) {
path_.clear();
/* Decode the parts of the ps into a vector. The first component of the
   vector will be empty if the ps starts with '/'. 
   We also refuse to append empty strings to path_. This handles cases like
   "/a//b" which will result in <"","a","b">.

  We use n0 to indicate the start position of a component and
  n1 to indicate the position just after the end of the component.
*/
string::size_type n0(ps.find_first_not_of('/',0));

if (n0!=0) // the path starts with '/' which we encode as a first "/" component.
  path_.push_back("");

if (n0==string::npos) // ps must be "/"
  return true;

string::size_type n1(string::npos);

/* The variable n0 points to the start of a component. We make n1 point to
   the next '/', if any. The component then is the substring starting at
   n0 and ending before n1.
*/
while ((n1=ps.find_first_of('/',n0))!=string::npos) {
  if (n1>n0) // only non-empty parts are stored.
    path_.push_back(ps.substr(n0,n1-n0));
  n0 = n1+1;
  }
// If ps does not end with '/', we must still add the final component.
if (n0<ps.size())
  path_.push_back(ps.substr(n0));

return true;
}

ostream& 
operator<<(ostream& os,const Path& path) {
// E.g. <"","a","b"> is printed as "/a/b".
for (unsigned int j=0; j<path.size() ; ++j) {
  if ((j>0)||(path.size()==1))
    os << "/";
  os << path[j];
  }
return os;
}

