#ifndef STATS_H
#define STATS_H
// $Id: stats.h,v 1.2 2001/04/12 15:26:19 dvermeir Exp $
#include <map>
#include <vector>
#include "configuration.h"
#include "logrecord.h"
#include "domain.h"
#include "path.h"
#include "datepattern.h"

//! This class represents the internal database of httpstats.
/* Selected logrecords can be added using Stats::add, the contents
   of the database can be printed using operator<<(ostream&,const Stats&).
*/
class Stats {
public:
  //! The configuration is copied locally.
  /*! It is used to decide what aspects of an added logrecord will be stored
     and what information will be printed.
  */
  Stats(const Configuration&);
  //! Add whatever is needed from a logrecord to the various Count maps (see below).
  /*! The decision on which map to add to depends on configuration_.
  */
  void add(const LogRecord&);

  //! Auxiliary functions for operator<<. 
  void print_dates(ostream&) const;
  //! Auxiliary functions for operator<<. 
  void print_paths(ostream&) const;
  //! Auxiliary functions for operator<<. 
  void print_domains(ostream&) const;

  //! Print stats information on ostream. 
  /*! What informatin (dates, paths or domains) depends on configuration_.
  */
  friend ostream&
  operator<<(ostream&,const Stats&);
private:
  //! The key of a DateCount map is a Date category vector.
  /*! E.g. <1999>, <1999,6>, <1999,6,3>, <1999,6,3,10> (June 3, 1999 10:00), ..
     The value is the number of logrecords in this category that have been added .
  */
  typedef map<DatePattern,unsigned int>         DateCount;
  //! The key of a PathCount map is a path category vector.
  /*! E.g. <""> ("/"), <"","a","b","c"> ("/a/b/c").
     The value is the number of logrecords in this category that have been added .
  */
  typedef map<Path,unsigned int>        PathCount;
  //! The key of a DomainCount map is a domain category vector.
  /*! <"be"> (*.be), <"be","ac","vub"> (*.vub.ac.be), <"be","ac","vub","tinf2">
      (tinf2.vub.ac.be).  The value is the number of logrecords in this category
      that have been added .
  */
  typedef map<Domain,unsigned int>      DomainCount;

  DateCount             dates_; 
  PathCount             paths_; 
  DomainCount           domains_;       
  //! Copy of constructor parameter.
  Configuration         configuration_; 
};
#endif
